@@ -29,6 +29,7 @@ from LbAPCommon.checks import (
     range_check_bkg_subtracted,
     range_check_nd,
 )
+from LbAPCommon.checks_utils import checks_to_JSON, hist_to_root
 
 from .utils import check_production
 
@@ -54,7 +55,7 @@ def perform_checks(production_name, job_name, test_ntuple_path_list, checks_outp
         check_results = run_checks(
             job_data["checks"], checks_data, test_ntuple_path_list
         )
-        output_check_results(checks_data, check_results, checks_output_dir)
+        output_check_results(job_name, checks_data, check_results, checks_output_dir)
 
         if all([check_results[cr].passed for cr in check_results]):
             click.secho(
@@ -179,16 +180,22 @@ def run_checks(checks_list, check_data, test_ntuple_path_list):
     return check_results
 
 
-def output_check_results(checks_data, check_results, checks_out_dir):
+def output_check_results(job_name, checks_data, check_results, checks_out_dir):
     """
     Handle all output from checks
     This includes both terminal output, and saving figures
     """
+    check_results_with_job = {job_name: check_results}
+    json_out_path = join(checks_out_dir, "checks.json")
+    checks_to_JSON(checks_data, check_results_with_job, json_output_path=json_out_path)
+
     for cr in check_results:
         if check_results[cr].passed:
             for key, data in check_results[cr].tree_data.items():
                 # Output histograms for 1D range
+                hist_counter = 0
                 for _histo in data.get("histograms", []):
+                    hist_counter += 1
                     fig, ax = plt.subplots(figsize=(10, 6))
                     if check_results[cr].check_type == "range":
                         mplhep.histplot(_histo)
@@ -199,12 +206,15 @@ def output_check_results(checks_data, check_results, checks_out_dir):
                         ax.set_xlabel(_histo.axes[0].name)
                         ax.set_ylabel("Frequency density")
                     elif check_results[cr].check_type == "range_nd":
-                        mplhep.hist2dplot(_histo)
-                        ax.set_xlabel(_histo.axes[0].name)
-                        ax.set_ylabel(_histo.axes[1].name)
+                        if len(_histo.axes) == 2:
+                            mplhep.hist2dplot(_histo)
+                            ax.set_xlabel(_histo.axes[0].name)
+                            ax.set_ylabel(_histo.axes[1].name)
                     tree_name = "_".join(key.split("/"))
-                    fig_name = f"{tree_name}_{cr}.pdf"
+                    fig_name = f"{job_name}_{tree_name}_{cr}_{hist_counter-1}.pdf"
                     plt.savefig(join(checks_out_dir, fig_name))
+            # Save histograms in a root file
+            hist_to_root(job_name, check_results[cr], checks_out_dir)
         else:
             click.secho(f"Check '{cr}' failed!", fg="red")
         for msg in check_results[cr].messages:
